searchData={"items":[{"type":"module","title":"BitcrowdEcto","doc":"This library contains Bitcrowd's collection of tiny Ecto helpers.\n\n* `BitcrowdEcto.Schema` is our default schema template which configures PK/FKs and timestamp types, among other things.\n* `BitcrowdEcto.Repo` contains extensions for Ecto repos, like `fetch/2`.\n* `BitcrowdEcto.Migrator` contains a tool for migrating from within releases.\n* `BitcrowdEcto.Migration` contains functions used in migrations.\n* `BitcrowdEcto.Changeset` contains mostly validators, plus the introspection-based `cast_all/3`.\n  - The `validate_money/3` validator is enabled when you add `ex_money` as a dependency to your project. Note that you need to recompile `bitcrowd_ecto` when you add `ex_money` later.\n* `BitcrowdEcto.DateTime` contains date/time helpers.\n* `BitcrowdEcto.Assertions` has an assortment of useful ExUnit assertions related to Ecto schemas.\n* `BitcrowdEcto.Random` contains functions that generate random tokens.","ref":"BitcrowdEcto.html"},{"type":"module","title":"BitcrowdEcto.Assertions","doc":"Useful little test assertions related to `t:Ecto.Changeset.t/0`.","ref":"BitcrowdEcto.Assertions.html"},{"type":"module","title":"Example - BitcrowdEcto.Assertions","doc":"Import this in your `ExUnit.CaseTemplate`:\n\n    defmodule MyApp.TestCase do\n      use ExUnit.CaseTemplate\n\n      using do\n        quote do\n          import Ecto\n          import Ecto.Changeset\n          import Ecto.Query\n          import BitcrowdEcto.Assertions\n        end\n      end\n    end","ref":"BitcrowdEcto.Assertions.html#module-example"},{"type":"function","title":"BitcrowdEcto.Assertions.assert_acceptance_error_on/2","doc":"Asserts that a changeset contains a failed \"acceptance\" validation on a given field.\n\nReturns the changeset for chainability.","ref":"BitcrowdEcto.Assertions.html#assert_acceptance_error_on/2"},{"type":"function","title":"BitcrowdEcto.Assertions.assert_almost_coincide/3","doc":"Allows to compare a DateTime field to another, testing whether they are roughly equal (d=5s).\nDelta defaults to 5 seconds and can be passed in optionally.","ref":"BitcrowdEcto.Assertions.html#assert_almost_coincide/3"},{"type":"function","title":"BitcrowdEcto.Assertions.assert_almost_now/1","doc":"Allows to compare a DateTime field to the present time.","ref":"BitcrowdEcto.Assertions.html#assert_almost_now/1"},{"type":"function","title":"BitcrowdEcto.Assertions.assert_cast_error_on/2","doc":"Asserts that a changeset contains a failed \"cast\" validation on a given field.\n\nReturns the changeset for chainability.","ref":"BitcrowdEcto.Assertions.html#assert_cast_error_on/2"},{"type":"function","title":"BitcrowdEcto.Assertions.assert_change_to_almost_now/2","doc":"Asserts that the value of a datetime field changed to the present time.","ref":"BitcrowdEcto.Assertions.html#assert_change_to_almost_now/2"},{"type":"function","title":"Example - BitcrowdEcto.Assertions.assert_change_to_almost_now/2","doc":"%TestSchema{datetime: nil}\n    |> Ecto.Changeset.change(%{datetime: DateTime.utc_now()})\n    |> assert_change_to_almost_now(:datetime)","ref":"BitcrowdEcto.Assertions.html#assert_change_to_almost_now/2-example"},{"type":"function","title":"BitcrowdEcto.Assertions.assert_changes/2","doc":"Asserts that a changeset contains a change of a given field.\n\nReturns the changeset for chainability.","ref":"BitcrowdEcto.Assertions.html#assert_changes/2"},{"type":"function","title":"BitcrowdEcto.Assertions.assert_changes/3","doc":"Asserts that a changeset contains a change of a given field to a given value.\n\nReturns the changeset for chainability.","ref":"BitcrowdEcto.Assertions.html#assert_changes/3"},{"type":"function","title":"BitcrowdEcto.Assertions.assert_changeset_valid/1","doc":"","ref":"BitcrowdEcto.Assertions.html#assert_changeset_valid/1"},{"type":"function","title":"BitcrowdEcto.Assertions.assert_count_difference/5","doc":"Assert that a given function changes the count of a given database table.","ref":"BitcrowdEcto.Assertions.html#assert_count_difference/5"},{"type":"function","title":"Example - BitcrowdEcto.Assertions.assert_count_difference/5","doc":"assert_count_difference Repo, Foo, 1, fn ->\n      Repo.insert(%Foo{})\n    end","ref":"BitcrowdEcto.Assertions.html#assert_count_difference/5-example"},{"type":"function","title":"BitcrowdEcto.Assertions.assert_count_differences/4","doc":"Assert multiple database table count changes.\n\nSee `assert_count_difference/5` for details.","ref":"BitcrowdEcto.Assertions.html#assert_count_differences/4"},{"type":"function","title":"Example - BitcrowdEcto.Assertions.assert_count_differences/4","doc":"assert_count_differences([{MyApp.Foo, 1}, {MyApp.Bar, -1}], fn ->\n      %MyApp.Foo{} |> MyApp.Repo.insert()\n      %MyApp.Bar{id: 1} |> MyApp.Repo.delete()\n    end","ref":"BitcrowdEcto.Assertions.html#assert_count_differences/4-example"},{"type":"function","title":"BitcrowdEcto.Assertions.assert_difference/4","doc":"Asserts that a given function changes the integer fetched by another function by a delta.","ref":"BitcrowdEcto.Assertions.html#assert_difference/4"},{"type":"function","title":"Example - BitcrowdEcto.Assertions.assert_difference/4","doc":"assert_difference fn -> Repo.count(Foo) end, 1 fn ->\n      %Foo{} |> Repo.insert()\n    end","ref":"BitcrowdEcto.Assertions.html#assert_difference/4-example"},{"type":"function","title":"BitcrowdEcto.Assertions.assert_error_on/4","doc":"Asserts that a changeset contains a given error on a given field.\n\nReturns the changeset for chainability.","ref":"BitcrowdEcto.Assertions.html#assert_error_on/4"},{"type":"function","title":"BitcrowdEcto.Assertions.assert_foreign_constraint_error_on/2","doc":"Asserts that a changeset contains a failed \"foreign_key\" constraint validation on a given field.\n\nReturns the changeset for chainability.","ref":"BitcrowdEcto.Assertions.html#assert_foreign_constraint_error_on/2"},{"type":"function","title":"BitcrowdEcto.Assertions.assert_foreign_key_constraint_error_on/2","doc":"Asserts that a changeset contains a failed \"foreign_key\" constraint validation on a given field.\n\nReturns the changeset for chainability.","ref":"BitcrowdEcto.Assertions.html#assert_foreign_key_constraint_error_on/2"},{"type":"function","title":"BitcrowdEcto.Assertions.assert_foreign_key_constraint_on/3","doc":"Asserts that a changeset contains a constraint on a given field.\n\nThis function looks into the changeset's (internal) `constraints` field to see if a\n`*_constraint` function has been called on it. Tests using this do not need to actually\nperform the database operation. However, given that the constraints only work in\ncombination with a corresponding database constraint, it is advisable to perform the\noperation and use `assert_foreign_key_constraint_error_on/2` instead.\n\nReturns the changeset for chainability.","ref":"BitcrowdEcto.Assertions.html#assert_foreign_key_constraint_on/3"},{"type":"function","title":"Options - BitcrowdEcto.Assertions.assert_foreign_key_constraint_on/3","doc":"The given options are used as match values against the constraint map. They loosely\ncorrespond to the options of `Ecto.Changeset.foreign_key_constraint/2`, only `:name`\nbecomes `:constraint`.","ref":"BitcrowdEcto.Assertions.html#assert_foreign_key_constraint_on/3-options"},{"type":"function","title":"BitcrowdEcto.Assertions.assert_format_error_on/2","doc":"Asserts that a changeset contains a failed \"format\" validation on a given field.\n\nReturns the changeset for chainability.","ref":"BitcrowdEcto.Assertions.html#assert_format_error_on/2"},{"type":"function","title":"BitcrowdEcto.Assertions.assert_inclusion_error_on/2","doc":"Asserts that a changeset contains a failed \"inclusion\" validation on a given field.\n\nReturns the changeset for chainability.","ref":"BitcrowdEcto.Assertions.html#assert_inclusion_error_on/2"},{"type":"function","title":"BitcrowdEcto.Assertions.assert_no_assoc_constraint_error_on/2","doc":"Asserts that a changeset contains a failed \"no_assoc\" constraint validation on a given field.\n\nReturns the changeset for chainability.","ref":"BitcrowdEcto.Assertions.html#assert_no_assoc_constraint_error_on/2"},{"type":"function","title":"BitcrowdEcto.Assertions.assert_no_assoc_constraint_on/3","doc":"Asserts that a changeset contains a constraint on a given field.\n\nThis function looks into the changeset's (internal) `constraints` field to see if a\n`*_constraint` function has been called on it. Tests using this do not need to actually\nperform the database operation. However, given that the constraints only work in\ncombination with a corresponding database constraint, it is advisable to perform the\noperation and use `assert_no_assoc_constraint_error_on/2` instead.\n\nReturns the changeset for chainability.","ref":"BitcrowdEcto.Assertions.html#assert_no_assoc_constraint_on/3"},{"type":"function","title":"Options - BitcrowdEcto.Assertions.assert_no_assoc_constraint_on/3","doc":"The given options are used as match values against the constraint map. They loosely\ncorrespond to the options of `Ecto.Changeset.no_assoc_constraint/2`, only `:name`\nbecomes `:constraint`.","ref":"BitcrowdEcto.Assertions.html#assert_no_assoc_constraint_on/3-options"},{"type":"function","title":"BitcrowdEcto.Assertions.assert_number_error_on/2","doc":"Asserts that a changeset contains a failed \"number\" validation on a given field.\n\nReturns the changeset for chainability.","ref":"BitcrowdEcto.Assertions.html#assert_number_error_on/2"},{"type":"function","title":"BitcrowdEcto.Assertions.assert_preloaded/2","doc":"Asserts that an Ecto struct has a preloaded nested struct at a given path.","ref":"BitcrowdEcto.Assertions.html#assert_preloaded/2"},{"type":"function","title":"BitcrowdEcto.Assertions.assert_required_error_on/2","doc":"Asserts that a changeset contains a failed \"required\" validation on a given field.\n\nReturns the changeset for chainability.","ref":"BitcrowdEcto.Assertions.html#assert_required_error_on/2"},{"type":"function","title":"BitcrowdEcto.Assertions.assert_sorted_equal/2","doc":"Assert that two lists are equal when sorted (Enum.sort).","ref":"BitcrowdEcto.Assertions.html#assert_sorted_equal/2"},{"type":"function","title":"Example - BitcrowdEcto.Assertions.assert_sorted_equal/2","doc":"assert_sorted_equal [:\"1\", :\"2\"], [:\"2\", :\"1\"]\n    assert_sorted_equal(\n      [%{id: 2}, %{id: 1}],\n      [%{id: 1, preload_nested_resource: %{id: 5}}, %{id: 2}],\n      & &1.id\n    )","ref":"BitcrowdEcto.Assertions.html#assert_sorted_equal/2-example"},{"type":"function","title":"BitcrowdEcto.Assertions.assert_sorted_equal/3","doc":"","ref":"BitcrowdEcto.Assertions.html#assert_sorted_equal/3"},{"type":"function","title":"BitcrowdEcto.Assertions.assert_unique_constraint_error_on/2","doc":"Asserts that a changeset contains a failed \"unique\" constraint validation on a given field.\n\nReturns the changeset for chainability.","ref":"BitcrowdEcto.Assertions.html#assert_unique_constraint_error_on/2"},{"type":"function","title":"BitcrowdEcto.Assertions.assert_unique_constraint_on/3","doc":"Asserts that a changeset contains a constraint on a given field.\n\nThis function looks into the changeset's (internal) `constraints` field to see if a\n`*_constraint` function has been called on it. Tests using this do not need to actually\nperform the database operation. However, given that the constraints only work in\ncombination with a corresponding database constraint, it is advisable to perform the\noperation and use `assert_unique_constraint_error_on/2` instead.\n\nReturns the changeset for chainability.","ref":"BitcrowdEcto.Assertions.html#assert_unique_constraint_on/3"},{"type":"function","title":"Options - BitcrowdEcto.Assertions.assert_unique_constraint_on/3","doc":"The given options are used as match values against the constraint map. They loosely\ncorrespond to the options of `Ecto.Changeset.unique_constraint/2`, only `:name`\nbecomes `:constraint`.","ref":"BitcrowdEcto.Assertions.html#assert_unique_constraint_on/3-options"},{"type":"function","title":"BitcrowdEcto.Assertions.flat_errors_on/3","doc":"A better error helper that transforms the errors on a given field into a list of\n`[ ,  ]`.\n\nIf multiple validations failed, the list will contain more elements! That simple.","ref":"BitcrowdEcto.Assertions.html#flat_errors_on/3"},{"type":"function","title":"Metadata - BitcrowdEcto.Assertions.flat_errors_on/3","doc":"By default, `flat_errors_on/2` extracts metadata from the `:validation` and `:constraint` keys,\nas those are were Ecto stores its metadata. Custom metadata at different keys can be extracted\nusing the `:metadata` option.","ref":"BitcrowdEcto.Assertions.html#flat_errors_on/3-metadata"},{"type":"function","title":"BitcrowdEcto.Assertions.refute_changes/2","doc":"Refutes that a changeset accepts changes to a given field.\n\nReturns the changeset for chainability.","ref":"BitcrowdEcto.Assertions.html#refute_changes/2"},{"type":"function","title":"BitcrowdEcto.Assertions.refute_changeset_valid/1","doc":"","ref":"BitcrowdEcto.Assertions.html#refute_changeset_valid/1"},{"type":"function","title":"BitcrowdEcto.Assertions.refute_difference/3","doc":"Assert that a given function doesn't change the value fetched by another function.","ref":"BitcrowdEcto.Assertions.html#refute_difference/3"},{"type":"function","title":"Example - BitcrowdEcto.Assertions.refute_difference/3","doc":"refute_difference fn -> Repo.count(Foo) end, fn ->\n      Repo.insert(%Foo{})\n    end","ref":"BitcrowdEcto.Assertions.html#refute_difference/3-example"},{"type":"function","title":"BitcrowdEcto.Assertions.refute_errors_on/2","doc":"Asserts that a changeset does not contain an error on a given field.\n\nReturns the changeset for chainability.","ref":"BitcrowdEcto.Assertions.html#refute_errors_on/2"},{"type":"function","title":"BitcrowdEcto.Assertions.refute_preloaded/2","doc":"Refutes that an Ecto struct has a preloaded nested struct at a given path.","ref":"BitcrowdEcto.Assertions.html#refute_preloaded/2"},{"type":"module","title":"BitcrowdEcto.Changeset","doc":"Extensions for Ecto changesets.","ref":"BitcrowdEcto.Changeset.html"},{"type":"function","title":"BitcrowdEcto.Changeset.cast_all/3","doc":"Introspects a schema and casts all defined fields from a params map.\n\n- Accepts a schema module or structs or changesets.\n- Can deal with embeds.","ref":"BitcrowdEcto.Changeset.html#cast_all/3"},{"type":"function","title":"Options - BitcrowdEcto.Changeset.cast_all/3","doc":"- `required` list of required field names\n- `optional` list of optional field names (= inverse set is required)\n\n`required` and `optional` options must not be present at the same time.","ref":"BitcrowdEcto.Changeset.html#cast_all/3-options"},{"type":"function","title":"Examples - BitcrowdEcto.Changeset.cast_all/3","doc":"iex> changeset = cast_all(TestEmbeddedSchema, %{some_field: 4})\n    iex> changeset.valid?\n    true\n\n    iex> changeset = cast_all(%TestEmbeddedSchema{}, %{some_field: 4})\n    iex> changeset.valid?\n    true\n\n    iex> changeset = cast_all(change(%TestEmbeddedSchema{}), %{some_field: 4})\n    iex> changeset.valid?\n    true\n\n    iex> changeset = cast_all(TestEmbeddedSchema, %{}, required: [:some_field])\n    iex> changeset.errors\n    [some_field: {\"can't be blank\", [validation: :required]}]\n\n    iex> changeset = cast_all(TestEmbeddedSchema, %{}, optional: [:some_other_field])\n    iex> changeset.errors\n    [some_field: {\"can't be blank\", [validation: :required]}]","ref":"BitcrowdEcto.Changeset.html#cast_all/3-examples"},{"type":"function","title":"BitcrowdEcto.Changeset.validate_changed/2","doc":"Validates that a field that has been changed.","ref":"BitcrowdEcto.Changeset.html#validate_changed/2"},{"type":"function","title":"BitcrowdEcto.Changeset.validate_date_after/4","doc":"Validates a date field in the changeset is after the given reference date.","ref":"BitcrowdEcto.Changeset.html#validate_date_after/4"},{"type":"function","title":"BitcrowdEcto.Changeset.validate_date_order/4","doc":"Validates two date fields to be a date range, so if both are set the first field has to be\nbefore the second field. The error is placed on the later field.","ref":"BitcrowdEcto.Changeset.html#validate_date_order/4"},{"type":"function","title":"Examples - BitcrowdEcto.Changeset.validate_date_order/4","doc":"validate_date_order(changeset, :from, :to)\n    validate_date_order(changeset, :from, :to, [valid_orders: :lt])\n    validate_date_order(changeset, :from, :to, [formatter: &Date.day_of_week/1])","ref":"BitcrowdEcto.Changeset.html#validate_date_order/4-examples"},{"type":"function","title":"BitcrowdEcto.Changeset.validate_datetime_after/4","doc":"Validates a field timestamp to be after the given one","ref":"BitcrowdEcto.Changeset.html#validate_datetime_after/4"},{"type":"function","title":"BitcrowdEcto.Changeset.validate_datetime_order/4","doc":"Validates two datetime fields to be a time range, so if both are set the first has to be before\nthe second field. The error is placed on the later field.","ref":"BitcrowdEcto.Changeset.html#validate_datetime_order/4"},{"type":"function","title":"Examples - BitcrowdEcto.Changeset.validate_datetime_order/4","doc":"validate_datetime_order(changeset, :from, :to)\n    validate_datetime_order(changeset, :from, :to, [valid_orders: :lt])\n    validate_datetime_order(changeset, :from, :to, [formatter: &DateTime.to_time/1])","ref":"BitcrowdEcto.Changeset.html#validate_datetime_order/4-examples"},{"type":"function","title":"BitcrowdEcto.Changeset.validate_email/3","doc":"Validates that an email has valid format.\n\n* Ignores nil values.","ref":"BitcrowdEcto.Changeset.html#validate_email/3"},{"type":"function","title":"Compliance - BitcrowdEcto.Changeset.validate_email/3","doc":"For a good list of valid/invalid emails, see https://gist.github.com/cjaoude/fd9910626629b53c4d25\n\nThe regex used in this validator doesn't understand half of the inputs, but we don't really care\nfor now. Validating super strange emails is not a sport we want to compete in.","ref":"BitcrowdEcto.Changeset.html#validate_email/3-compliance"},{"type":"function","title":"Options - BitcrowdEcto.Changeset.validate_email/3","doc":"* `:max_length` - restricts the maximum length of the input, defaults to 320\n* `:only_web` - requires a dot in the domain part, e.g. `domain.tld`, defaults to true","ref":"BitcrowdEcto.Changeset.html#validate_email/3-options"},{"type":"function","title":"BitcrowdEcto.Changeset.validate_future_datetime/3","doc":"Validates a field timestamp to be in the future, if present","ref":"BitcrowdEcto.Changeset.html#validate_future_datetime/3"},{"type":"function","title":"BitcrowdEcto.Changeset.validate_hex_color/2","doc":"Validates a changeset field with hexadecimal color format","ref":"BitcrowdEcto.Changeset.html#validate_hex_color/2"},{"type":"function","title":"BitcrowdEcto.Changeset.validate_immutable/2","doc":"Validates that a field is not changed from its current value, unless the current value is nil.","ref":"BitcrowdEcto.Changeset.html#validate_immutable/2"},{"type":"function","title":"BitcrowdEcto.Changeset.validate_money/3","doc":"Validates a `t:Money.t` value according to the given options.","ref":"BitcrowdEcto.Changeset.html#validate_money/3"},{"type":"function","title":"Examples - BitcrowdEcto.Changeset.validate_money/3","doc":"validate_money(changeset, :amount, less_than: Money.new(\"100.00\", :USD))\n    validate_money(changeset, :amount, greater_than_or_equal_to: Money.new(\"100.00\", :USD))\n    validate_money(changeset, :amount, currency: :USD)","ref":"BitcrowdEcto.Changeset.html#validate_money/3-examples"},{"type":"function","title":"BitcrowdEcto.Changeset.validate_order/5","doc":"Validates two fields to be a range, so if both are set the first has to be before\nthe second field. The error is placed on the second field.","ref":"BitcrowdEcto.Changeset.html#validate_order/5"},{"type":"function","title":"Examples - BitcrowdEcto.Changeset.validate_order/5","doc":"validate_order(changeset, :from, :to, :to_is_after_from)\n    validate_order(changeset, :from, :to, :to_is_after_from, [compare_fun: fn a, b -> String.length(a) > String.length(b) end])\n    validate_order(changeset, :from, :to, :to_is_after_from, [formatter: &String.length/1])","ref":"BitcrowdEcto.Changeset.html#validate_order/5-examples"},{"type":"function","title":"BitcrowdEcto.Changeset.validate_past_datetime/3","doc":"Validates a field timestamp to be in the past, if present","ref":"BitcrowdEcto.Changeset.html#validate_past_datetime/3"},{"type":"function","title":"BitcrowdEcto.Changeset.validate_transition/3","doc":"Validates that a field has changed in a defined way.","ref":"BitcrowdEcto.Changeset.html#validate_transition/3"},{"type":"function","title":"Examples - BitcrowdEcto.Changeset.validate_transition/3","doc":"validate_transition(changeset, field, [{\"foo\", \"bar\"}, {\"foo\", \"yolo\"}])\n\nThis marks the changeset invalid unless the value of `:field` is currently `\"foo\"` and is\nchanged to `\"bar\"` or `\"yolo\"`. If the field is not changed, a `{state, state}` transition\nhas to be present in the list of transitions.","ref":"BitcrowdEcto.Changeset.html#validate_transition/3-examples"},{"type":"function","title":"BitcrowdEcto.Changeset.validate_url/2","doc":"Validates a field url to be qualified url","ref":"BitcrowdEcto.Changeset.html#validate_url/2"},{"type":"type","title":"BitcrowdEcto.Changeset.cast_all_option/0","doc":"","ref":"BitcrowdEcto.Changeset.html#t:cast_all_option/0"},{"type":"type","title":"BitcrowdEcto.Changeset.validate_email_option/0","doc":"","ref":"BitcrowdEcto.Changeset.html#t:validate_email_option/0"},{"type":"type","title":"BitcrowdEcto.Changeset.validate_money_option/0","doc":"","ref":"BitcrowdEcto.Changeset.html#t:validate_money_option/0"},{"type":"module","title":"BitcrowdEcto.DateTime","doc":"Functions to work with date and time values.","ref":"BitcrowdEcto.DateTime.html"},{"type":"function","title":"BitcrowdEcto.DateTime.beginning_of_day/1","doc":"Works similar to `Timex.beginning_of_day/3`, but way more simple.","ref":"BitcrowdEcto.DateTime.html#beginning_of_day/1"},{"type":"function","title":"Behaviour - BitcrowdEcto.DateTime.beginning_of_day/1","doc":"Nulls the time fields of the `DateTime` and keeps the rest.","ref":"BitcrowdEcto.DateTime.html#beginning_of_day/1-behaviour"},{"type":"function","title":"Examples - BitcrowdEcto.DateTime.beginning_of_day/1","doc":"iex> beginning_of_day(~U[2022-04-07 07:21:22.036Z])\n    ~U[2022-04-07 00:00:00.000000Z]","ref":"BitcrowdEcto.DateTime.html#beginning_of_day/1-examples"},{"type":"function","title":"BitcrowdEcto.DateTime.beginning_of_last_month/1","doc":"Calculates the beginning of last month.","ref":"BitcrowdEcto.DateTime.html#beginning_of_last_month/1"},{"type":"function","title":"Behaviour - BitcrowdEcto.DateTime.beginning_of_last_month/1","doc":"Goes to this month's beginning, subtracts 15 days, and goes back to the month's beginning.\n\nShould be relatively safe.","ref":"BitcrowdEcto.DateTime.html#beginning_of_last_month/1-behaviour"},{"type":"function","title":"Examples - BitcrowdEcto.DateTime.beginning_of_last_month/1","doc":"iex> beginning_of_last_month(~U[2022-04-07 07:21:22.036Z])\n    ~U[2022-03-01 00:00:00.000000Z]\n\n    iex> beginning_of_last_month(~U[2022-02-07 07:21:22.036Z])\n    ~U[2022-01-01 00:00:00.000000Z]","ref":"BitcrowdEcto.DateTime.html#beginning_of_last_month/1-examples"},{"type":"function","title":"BitcrowdEcto.DateTime.beginning_of_month/1","doc":"Works similar to `Timex.beginning_of_day/3`, but way more simple.","ref":"BitcrowdEcto.DateTime.html#beginning_of_month/1"},{"type":"function","title":"Behaviour - BitcrowdEcto.DateTime.beginning_of_month/1","doc":"Sets `day` to 1 and nulls the time fields.","ref":"BitcrowdEcto.DateTime.html#beginning_of_month/1-behaviour"},{"type":"function","title":"Examples - BitcrowdEcto.DateTime.beginning_of_month/1","doc":"iex> beginning_of_month(~U[2022-04-07 07:21:22.036Z])\n    ~U[2022-04-01 00:00:00.000000Z]","ref":"BitcrowdEcto.DateTime.html#beginning_of_month/1-examples"},{"type":"function","title":"BitcrowdEcto.DateTime.beginning_of_next_month/1","doc":"Calculates the beginning of next month.","ref":"BitcrowdEcto.DateTime.html#beginning_of_next_month/1"},{"type":"function","title":"Behaviour - BitcrowdEcto.DateTime.beginning_of_next_month/1","doc":"Goes to this month's beginning, adds 45 days, and goes back to the month's beginning.\n\nShould be relatively safe.","ref":"BitcrowdEcto.DateTime.html#beginning_of_next_month/1-behaviour"},{"type":"function","title":"Examples - BitcrowdEcto.DateTime.beginning_of_next_month/1","doc":"iex> beginning_of_next_month(~U[2022-04-07 07:21:22.036Z])\n    ~U[2022-05-01 00:00:00.000000Z]\n\n    iex> beginning_of_next_month(~U[2022-02-07 07:21:22.036Z])\n    ~U[2022-03-01 00:00:00.000000Z]","ref":"BitcrowdEcto.DateTime.html#beginning_of_next_month/1-examples"},{"type":"function","title":"BitcrowdEcto.DateTime.beginning_of_tomorrow/1","doc":"Calculates the beginning of tomorrow, equalizing day length differences due to DST.","ref":"BitcrowdEcto.DateTime.html#beginning_of_tomorrow/1"},{"type":"function","title":"Behaviour - BitcrowdEcto.DateTime.beginning_of_tomorrow/1","doc":"Adds 1.5d to today's midnight and goes back to midnight. Should be relatively safe.","ref":"BitcrowdEcto.DateTime.html#beginning_of_tomorrow/1-behaviour"},{"type":"function","title":"Examples - BitcrowdEcto.DateTime.beginning_of_tomorrow/1","doc":"iex> beginning_of_tomorrow(~U[2022-04-07 07:21:22.036Z])\n    ~U[2022-04-08 00:00:00.000000Z]\n\n    iex> ~U[2020-03-29 12:00:00.000Z]\n    ...> |> DateTime.shift_zone!(\"Europe/Berlin\")\n    ...> |> beginning_of_tomorrow()\n    ...> |> DateTime.to_iso8601()\n    \"2020-03-30T00:00:00.000000+02:00\"","ref":"BitcrowdEcto.DateTime.html#beginning_of_tomorrow/1-examples"},{"type":"function","title":"BitcrowdEcto.DateTime.beginning_of_yesterday/1","doc":"Calculates the beginning of yesterday, equalizing day length differences due to DST.","ref":"BitcrowdEcto.DateTime.html#beginning_of_yesterday/1"},{"type":"function","title":"Behaviour - BitcrowdEcto.DateTime.beginning_of_yesterday/1","doc":"Subtracts 0.5d from today's midnight and goes back to midnight. Should be relatively safe.","ref":"BitcrowdEcto.DateTime.html#beginning_of_yesterday/1-behaviour"},{"type":"function","title":"Examples - BitcrowdEcto.DateTime.beginning_of_yesterday/1","doc":"iex> beginning_of_yesterday(~U[2022-04-07 07:21:22.036Z])\n    ~U[2022-04-06 00:00:00.000000Z]\n\n    iex> ~U[2020-03-29 12:00:00.000Z]\n    ...> |> DateTime.shift_zone!(\"Europe/Berlin\")\n    ...> |> beginning_of_yesterday()\n    ...> |> DateTime.to_iso8601()\n    \"2020-03-28T00:00:00.000000+01:00\"","ref":"BitcrowdEcto.DateTime.html#beginning_of_yesterday/1-examples"},{"type":"function","title":"BitcrowdEcto.DateTime.in_seconds/1","doc":"Converts a `{ ,  }` tuple into seconds.\n\n#Examples\n\n    iex> in_seconds({99, :second})\n    99\n\n    iex> in_seconds({1, :minute})\n    60\n\n    iex> in_seconds({1, :hour})\n    3600\n\n    iex> in_seconds({1, :day})\n    86400\n\n    iex> in_seconds({1, :week})\n    604800","ref":"BitcrowdEcto.DateTime.html#in_seconds/1"},{"type":"function","title":"BitcrowdEcto.DateTime.shift/2","doc":"Works similar to `Timex.shift/3`, but way more simple.","ref":"BitcrowdEcto.DateTime.html#shift/2"},{"type":"function","title":"Behaviour - BitcrowdEcto.DateTime.shift/2","doc":"Semantics are like `DateTime.add/3`. TimeZone-awareness when using tzdata.\nDateTime, e.g. \"2020-03-29 14:00 Europe/Berlin\" - 1 day = \"2020-03-28 13:00\" as March 29th\nonly had 23 hours due to DST.","ref":"BitcrowdEcto.DateTime.html#shift/2-behaviour"},{"type":"function","title":"Examples - BitcrowdEcto.DateTime.shift/2","doc":"iex> shift(~U[2022-04-07 07:21:22.036Z], 15)\n    ~U[2022-04-07 07:21:37.036Z]\n\n    iex> shift(~U[2022-04-07 07:21:22.036Z], -3600)\n    ~U[2022-04-07 06:21:22.036Z]\n\n    iex> shift(~U[2022-04-07 07:21:22.036Z], {1, :day})\n    ~U[2022-04-08 07:21:22.036Z]\n\n    iex> ~U[2020-03-29 12:00:00.000Z]\n    ...> |> DateTime.shift_zone!(\"Europe/Berlin\")\n    ...> |> shift({-1, :day})\n    ...> |> DateTime.to_iso8601()\n    \"2020-03-28T13:00:00.000+01:00\"","ref":"BitcrowdEcto.DateTime.html#shift/2-examples"},{"type":"type","title":"BitcrowdEcto.DateTime.period/0","doc":"","ref":"BitcrowdEcto.DateTime.html#t:period/0"},{"type":"type","title":"BitcrowdEcto.DateTime.unit/0","doc":"","ref":"BitcrowdEcto.DateTime.html#t:unit/0"},{"type":"module","title":"BitcrowdEcto.Migration","doc":"Utilities for migrations.","ref":"BitcrowdEcto.Migration.html"},{"type":"function","title":"BitcrowdEcto.Migration.grant_dml_privileges_on_schema/3","doc":"This function grants data manipulation privileges for a given schema to a given role.\n\nUse this when you have a setup where your \"runtime\" user is deprived of all DDL privileges\n(i.e., it can't create or modify tables, etc.), but your migration user is allowed to do so.\n\nIn particular, the following GRANTS are executed:\n\n    GRANT SELECT, INSERT, UPDATE, DELETE ON ALL TABLES\n    GRANT ALL ON ALL FUNCTIONS\n    GRANT ALL ON ALL SEQUENCES","ref":"BitcrowdEcto.Migration.html#grant_dml_privileges_on_schema/3"},{"type":"function","title":"Usage - BitcrowdEcto.Migration.grant_dml_privileges_on_schema/3","doc":"The easiest way of using this is to issue the command after every `CREATE SCHEMA` call:\n\n    def change do\n      execute(\"CREATE SCHEMA foo;\", \"DROP SCHEMA foo;\")\n\n      if direction() == :up do\n        BitcrowdEcto.Migration.grant_dml_privileges_on_schema(\"foo\", \"mydmlrole\")\n      end\n    end","ref":"BitcrowdEcto.Migration.html#grant_dml_privileges_on_schema/3-usage"},{"type":"function","title":"Options - BitcrowdEcto.Migration.grant_dml_privileges_on_schema/3","doc":"* `default`  boolean indicating whether the privileges should automatically be granted for\n             future objects, defaults to true","ref":"BitcrowdEcto.Migration.html#grant_dml_privileges_on_schema/3-options"},{"type":"behaviour","title":"BitcrowdEcto.Migrator","doc":"Release migration logic for repositories.\n\nCan deal with normal repositories & multi-tenant repositories.","ref":"BitcrowdEcto.Migrator.html"},{"type":"behaviour","title":"Usage - BitcrowdEcto.Migrator","doc":"In the simplest case, add the Migrator to your repository:\n\n    defmodule MyApp.Repo do\n      use Ecto.Repo, otp_app: :my_app, adapter: Ecto.Adapters.Postgres\n\n      use BitcrowdEcto.Migrator\n    end\n\nand call the migrator from your release:\n\n    $ bin/my_app eval 'MyApp.Repo.up()'\n\n#","ref":"BitcrowdEcto.Migrator.html#module-usage"},{"type":"behaviour","title":"Multi-Tenant repositories - BitcrowdEcto.Migrator","doc":"For multi-tenant repositories, you need to provide a list of tenants (= PG schemas) by\noverriding the `known_prefixes/0` function on your repository:\n\n    defmodule MyApp.Repo do\n      use Ecto.Repo, otp_app: :my_app, adapter: Ecto.Adapters.Postgres\n\n      use BitcrowdEcto.Migrator\n\n      def known_prefixes do\n        [\"tenant_a\", \"tenant_b\"]\n      end\n    end\n\nThis will make the migrator apply migrations from the `priv/repo/tenant_migrations` directory\nonto schemas `tenant_a` and `tenant_b`. The schemas will be created if necessary.\n\n#","ref":"BitcrowdEcto.Migrator.html#module-multi-tenant-repositories"},{"type":"behaviour","title":"Mix tasks for development - BitcrowdEcto.Migrator","doc":"In normal development without multi-tenancy, the usual Ecto mix tasks will work just fine.\n\nWhen using tenant schemas, the normal Ecto mix tasks will only apply the \"global\" (i.e.\nnon-prefixed migrations) from `priv/repo/migrations`, which is not enough. You can define\nyour own Mix tasks calling the `up/0` and `down/1` functions on your repository:\n\n    defmodule Mix.Tasks.MyApp.Migrate do\n      use Mix.Task\n\n      @shortdoc \"Migrates our repository\"\n      @moduledoc \"Migrates the repository including the tenant schemas\"\n\n      @impl true\n      def run(args) do\n        Mix.Task.run(\"app.config\", args)\n\n        MyApp.Repo.up()\n      end\n    end\n\n    defmodule Mix.Tasks.MyApp.Rollback do\n      use Mix.Task\n\n      @shortdoc \"Rolls back our repository\"\n      @moduledoc \"Rolls back the repository including the tenant schemas\"\n\n      @impl true\n      def run(args) do\n        Mix.Task.run(\"app.config\", args)\n\n        {[to: to], _} = OptionParser.parse!(args, strict: [to: :integer], aliases: [])\n\n        MyApp.Repo.down(to)\n      end\n    end\n\n#","ref":"BitcrowdEcto.Migrator.html#module-mix-tasks-for-development"},{"type":"behaviour","title":"Stopping the application if migrations are down - BitcrowdEcto.Migrator","doc":"In some setups (e.g. parallel starts of Kubernetes jobs for migrations and your app workload)\nyou may want to abort startup if migrations haven't been applied. Use the `ensure_up!/0`\nfunction to raise an exception in such cases.\n\n    $ bin/my_app eval 'MyApp.Repo.ensure_up!()'\n    [...]\n    ** (RuntimeError) Migrations not up!\n\n      20220101120000_create_foo_table.exs\n    [...]\n\n    $ echo $?\n    1\n\n    $ bin/my_app eval 'MyApp.Repo.up()'\n    [...]\n\n    $ bin/my_app eval 'MyApp.Repo.ensure_up!()'\n    [...]\n\n    $ echo $?\n    0","ref":"BitcrowdEcto.Migrator.html#module-stopping-the-application-if-migrations-are-down"},{"type":"callback","title":"BitcrowdEcto.Migrator.down/1","doc":"Rolls back both the main schemas/tables and the tenant schemas to a given version.","ref":"BitcrowdEcto.Migrator.html#c:down/1"},{"type":"callback","title":"BitcrowdEcto.Migrator.ensure_up!/0","doc":"Checks that all migrations have been run or raises an exception otherwise.","ref":"BitcrowdEcto.Migrator.html#c:ensure_up!/0"},{"type":"callback","title":"BitcrowdEcto.Migrator.handle_migrator_exception/2","doc":"Called when the migrator experiences an exception.","ref":"BitcrowdEcto.Migrator.html#c:handle_migrator_exception/2"},{"type":"callback","title":"BitcrowdEcto.Migrator.known_prefixes/0","doc":"Returns the list of prefixes used on this repository.","ref":"BitcrowdEcto.Migrator.html#c:known_prefixes/0"},{"type":"callback","title":"BitcrowdEcto.Migrator.up/0","doc":"Migrates both the \"main\" (i.e. non-tenant) schemas/tables and the tenant schemas to their\nlatest version.","ref":"BitcrowdEcto.Migrator.html#c:up/0"},{"type":"module","title":"BitcrowdEcto.Random","doc":"Various random value generators.","ref":"BitcrowdEcto.Random.html"},{"type":"function","title":"BitcrowdEcto.Random.unambiguous_human_token/1","doc":"Generates a random token from an alphabet designed to have no ambiguities.","ref":"BitcrowdEcto.Random.html#unambiguous_human_token/1"},{"type":"function","title":"BitcrowdEcto.Random.url_token/1","doc":"Generates a random token suitable for inclusion in URLs.","ref":"BitcrowdEcto.Random.html#url_token/1"},{"type":"function","title":"BitcrowdEcto.Random.uuid/0","doc":"Generates a UUID V4.","ref":"BitcrowdEcto.Random.html#uuid/0"},{"type":"behaviour","title":"BitcrowdEcto.Repo","doc":"Extensions for Ecto repos.","ref":"BitcrowdEcto.Repo.html"},{"type":"behaviour","title":"Usage - BitcrowdEcto.Repo","doc":"defmodule MyApp.Repo do\n      use Ecto.Repo, otp_app: :my_app, adapter: Ecto.Adapters.Postgres\n\n      use BitcrowdEcto.Repo\n    end","ref":"BitcrowdEcto.Repo.html#module-usage"},{"type":"callback","title":"BitcrowdEcto.Repo.advisory_xact_lock/1","doc":"Acquires an advisory lock for a named resource.\n\nAdvisory locks are helpful when you don't have a specific row to lock, but also don't want\nto lock an entire table.\n\nSee https://www.postgresql.org/docs/9.4/explicit-locking.html#ADVISORY-LOCKS","ref":"BitcrowdEcto.Repo.html#c:advisory_xact_lock/1"},{"type":"callback","title":"Example - BitcrowdEcto.Repo.advisory_xact_lock/1","doc":"MyApp.Repo.transaction(fn ->\n      MyApp.Repo.advisory_lock(:foo)\n      # Advisory lock is held until end of transaction\n    end)","ref":"BitcrowdEcto.Repo.html#c:advisory_xact_lock/1-example"},{"type":"callback","title":"A note on the advisory lock key - BitcrowdEcto.Repo.advisory_xact_lock/1","doc":"`pg_advisory_xact_lock()` has two versions: One which you pass a 64-bit signed integer as\nthe lock key, and one which you pass two 32-bit integers as the keys (e.g., one\n\"application\" key and one specific lock key), in which case PostgreSQL concatenates them\ninto a 64-bit value. In any case you need to pass integers.\n\nWe decided that we wanted to have atom or string keys for better readability.  Hence, in=\norder to make PostgreSQL happy, we hash these strings into 64 bits signed ints.\n\n64 bits make a pretty big number already, so it is quite unlikely that two of our keys (or\nkeys of other libraries that use advisory locks, e.g. Oban) collide. But for an extra false\nsense of safety we use a crypto hash algorithm to ensure that keys spread out over the\ndomain uniformly. Luckily, taking a prefix of a cryptographic hash does not break its\nuniformity.","ref":"BitcrowdEcto.Repo.html#c:advisory_xact_lock/1-a-note-on-the-advisory-lock-key"},{"type":"callback","title":"BitcrowdEcto.Repo.count/1","doc":"Allows to conveniently count a queryable.\n\nSee `c:count/2` for options.","ref":"BitcrowdEcto.Repo.html#c:count/1"},{"type":"callback","title":"BitcrowdEcto.Repo.count/2","doc":"Allows to conveniently count a queryable.","ref":"BitcrowdEcto.Repo.html#c:count/2"},{"type":"callback","title":"Ecto options - BitcrowdEcto.Repo.count/2","doc":"* `prefix`             See https://hexdocs.pm/ecto/Ecto.Repo.html#c:one/2-options\n* `timeout`            See [Ecto's Shared Options](https://hexdocs.pm/ecto/Ecto.Repo.html#module-shared-options)\n* `log`                See [Ecto's Shared Options](https://hexdocs.pm/ecto/Ecto.Repo.html#module-shared-options)\n* `telemetry_event`    See [Ecto's Shared Options](https://hexdocs.pm/ecto/Ecto.Repo.html#module-shared-options)\n* `telemetry_options`  See [Ecto's Shared Options](https://hexdocs.pm/ecto/Ecto.Repo.html#module-shared-options)","ref":"BitcrowdEcto.Repo.html#c:count/2-ecto-options"},{"type":"callback","title":"BitcrowdEcto.Repo.fetch/2","doc":"Fetches a record by primary key or returns a \"tagged\" error tuple.\n\nSee `c:fetch_by/3`.","ref":"BitcrowdEcto.Repo.html#c:fetch/2"},{"type":"callback","title":"BitcrowdEcto.Repo.fetch/3","doc":"Fetches a record by given clauses or returns a \"tagged\" error tuple.\n\nSee `c:fetch_by/3` for options.","ref":"BitcrowdEcto.Repo.html#c:fetch/3"},{"type":"callback","title":"BitcrowdEcto.Repo.fetch_by/2","doc":"Fetches a record by given clauses or returns a \"tagged\" error tuple.\n\nSee `c:fetch_by/3` for options.","ref":"BitcrowdEcto.Repo.html#c:fetch_by/2"},{"type":"callback","title":"BitcrowdEcto.Repo.fetch_by/3","doc":"Fetches a record by given clauses or returns a \"tagged\" error tuple.\n\n- On success, the record is wrapped in a `:ok` tuple.\n- On error, a \"tagged\" error tuple is returned that contains the *original* queryable or module\n  as the tag, e.g. `{:error, {:not_found, Account}}` for a `fetch_by(Account, id: 1)` call.\n\nPassing invalid values that would normally result in an `Ecto.Query.CastError` will result in\na `:not_found` error tuple as well.\n\nThis function can also apply row locks.","ref":"BitcrowdEcto.Repo.html#c:fetch_by/3"},{"type":"callback","title":"Options - BitcrowdEcto.Repo.fetch_by/3","doc":"* `lock`               any of `[:no_key_update, :update]` (defaults to `false`)\n* `preload`            allows to preload associations\n* `error_tag`          allows to specify a custom \"tag\" value (instead of the queryable)\n* `raise_cast_error`   raise `CastError` instead of converting to `:not_found` (defaults to `false`)","ref":"BitcrowdEcto.Repo.html#c:fetch_by/3-options"},{"type":"callback","title":"Ecto options - BitcrowdEcto.Repo.fetch_by/3","doc":"* `prefix`             See https://hexdocs.pm/ecto/Ecto.Repo.html#c:one/2-options\n* `timeout`            See [Ecto's Shared Options](https://hexdocs.pm/ecto/Ecto.Repo.html#module-shared-options)\n* `log`                See [Ecto's Shared Options](https://hexdocs.pm/ecto/Ecto.Repo.html#module-shared-options)\n* `telemetry_event`    See [Ecto's Shared Options](https://hexdocs.pm/ecto/Ecto.Repo.html#module-shared-options)\n* `telemetry_options`  See [Ecto's Shared Options](https://hexdocs.pm/ecto/Ecto.Repo.html#module-shared-options)","ref":"BitcrowdEcto.Repo.html#c:fetch_by/3-ecto-options"},{"type":"type","title":"BitcrowdEcto.Repo.ecto_option/0","doc":"","ref":"BitcrowdEcto.Repo.html#t:ecto_option/0"},{"type":"type","title":"BitcrowdEcto.Repo.fetch_option/0","doc":"","ref":"BitcrowdEcto.Repo.html#t:fetch_option/0"},{"type":"type","title":"BitcrowdEcto.Repo.fetch_result/0","doc":"","ref":"BitcrowdEcto.Repo.html#t:fetch_result/0"},{"type":"type","title":"BitcrowdEcto.Repo.lock_mode/0","doc":"","ref":"BitcrowdEcto.Repo.html#t:lock_mode/0"},{"type":"module","title":"BitcrowdEcto.Schema","doc":"An opinionated set of defaults for Ecto schemas.\n\n* Uses `Ecto.Schema` and imports `Ecto.Changeset` and `BitcrowdEcto.Changeset`\n* Configures an autogenerated PK of type `binary_id`\n* Configures FKs to be of type `binary_id`\n* Sets timestamp type to `utc_datetime_usec`\n* Defines a type `t` as a struct of the schema module.\n* Defines an `id` type","ref":"BitcrowdEcto.Schema.html"},{"type":"module","title":"Usage - BitcrowdEcto.Schema","doc":"defmodule MyApp.MySchema do\n      use BitcrowdEcto.Schema\n    end\n\nOr if you table lives in a different Postgres schema:\n\n    defmodule MyApp.MySchema do\n      use BitcrowdEcto.Schema, prefix: \"foo\"\n    end","ref":"BitcrowdEcto.Schema.html#module-usage"},{"type":"function","title":"BitcrowdEcto.Schema.to_enum_member/3","doc":"Safely converts a string into an enum member atom. Returns nil if conversion is not posssible.","ref":"BitcrowdEcto.Schema.html#to_enum_member/3"},{"type":"function","title":"Example - BitcrowdEcto.Schema.to_enum_member/3","doc":"iex> to_enum_member(TestEnumSchema, :some_enum, \"foo\")\n    :foo","ref":"BitcrowdEcto.Schema.html#to_enum_member/3-example"},{"type":"function","title":"BitcrowdEcto.Schema.to_enum_member!/3","doc":"Safely converts a string into an enum member atom. Raises if conversion is not possible.","ref":"BitcrowdEcto.Schema.html#to_enum_member!/3"},{"type":"extras","title":"LICENSE","doc":"Apache License\n                           Version 2.0, January 2004\n                        http://www.apache.org/licenses/\n\n   TERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION\n\n   1. Definitions.\n\n      \"License\" shall mean the terms and conditions for use, reproduction,\n      and distribution as defined by Sections 1 through 9 of this document.\n\n      \"Licensor\" shall mean the copyright owner or entity authorized by\n      the copyright owner that is granting the License.\n\n      \"Legal Entity\" shall mean the union of the acting entity and all\n      other entities that control, are controlled by, or are under common\n      control with that entity. For the purposes of this definition,\n      \"control\" means (i) the power, direct or indirect, to cause the\n      direction or management of such entity, whether by contract or\n      otherwise, or (ii) ownership of fifty percent (50%) or more of the\n      outstanding shares, or (iii) beneficial ownership of such entity.\n\n      \"You\" (or \"Your\") shall mean an individual or Legal Entity\n      exercising permissions granted by this License.\n\n      \"Source\" form shall mean the preferred form for making modifications,\n      including but not limited to software source code, documentation\n      source, and configuration files.\n\n      \"Object\" form shall mean any form resulting from mechanical\n      transformation or translation of a Source form, including but\n      not limited to compiled object code, generated documentation,\n      and conversions to other media types.\n\n      \"Work\" shall mean the work of authorship, whether in Source or\n      Object form, made available under the License, as indicated by a\n      copyright notice that is included in or attached to the work\n      (an example is provided in the Appendix below).\n\n      \"Derivative Works\" shall mean any work, whether in Source or Object\n      form, that is based on (or derived from) the Work and for which the\n      editorial revisions, annotations, elaborations, or other modifications\n      represent, as a whole, an original work of authorship. For the purposes\n      of this License, Derivative Works shall not include works that remain\n      separable from, or merely link (or bind by name) to the interfaces of,\n      the Work and Derivative Works thereof.\n\n      \"Contribution\" shall mean any work of authorship, including\n      the original version of the Work and any modifications or additions\n      to that Work or Derivative Works thereof, that is intentionally\n      submitted to Licensor for inclusion in the Work by the copyright owner\n      or by an individual or Legal Entity authorized to submit on behalf of\n      the copyright owner. For the purposes of this definition, \"submitted\"\n      means any form of electronic, verbal, or written communication sent\n      to the Licensor or its representatives, including but not limited to\n      communication on electronic mailing lists, source code control systems,\n      and issue tracking systems that are managed by, or on behalf of, the\n      Licensor for the purpose of discussing and improving the Work, but\n      excluding communication that is conspicuously marked or otherwise\n      designated in writing by the copyright owner as \"Not a Contribution.\"\n\n      \"Contributor\" shall mean Licensor and any individual or Legal Entity\n      on behalf of whom a Contribution has been received by Licensor and\n      subsequently incorporated within the Work.\n\n   2. Grant of Copyright License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      copyright license to reproduce, prepare Derivative Works of,\n      publicly display, publicly perform, sublicense, and distribute the\n      Work and such Derivative Works in Source or Object form.\n\n   3. Grant of Patent License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      (except as stated in this section) patent license to make, have made,\n      use, offer to sell, sell, import, and otherwise transfer the Work,\n      where such license applies only to those patent claims licensable\n      by such Contributor that are necessarily infringed by their\n      Contribution(s) alone or by combination of their Contribution(s)\n      with the Work to which such Contribution(s) was submitted. If You\n      institute patent litigation against any entity (including a\n      cross-claim or counterclaim in a lawsuit) alleging that the Work\n      or a Contribution incorporated within the Work constitutes direct\n      or contributory patent infringement, then any patent licenses\n      granted to You under this License for that Work shall terminate\n      as of the date such litigation is filed.\n\n   4. Redistribution. You may reproduce and distribute copies of the\n      Work or Derivative Works thereof in any medium, with or without\n      modifications, and in Source or Object form, provided that You\n      meet the following conditions:\n\n      (a) You must give any other recipients of the Work or\n          Derivative Works a copy of this License; and\n\n      (b) You must cause any modified files to carry prominent notices\n          stating that You changed the files; and\n\n      (c) You must retain, in the Source form of any Derivative Works\n          that You distribute, all copyright, patent, trademark, and\n          attribution notices from the Source form of the Work,\n          excluding those notices that do not pertain to any part of\n          the Derivative Works; and\n\n      (d) If the Work includes a \"NOTICE\" text file as part of its\n          distribution, then any Derivative Works that You distribute must\n          include a readable copy of the attribution notices contained\n          within such NOTICE file, excluding those notices that do not\n          pertain to any part of the Derivative Works, in at least one\n          of the following places: within a NOTICE text file distributed\n          as part of the Derivative Works; within the Source form or\n          documentation, if provided along with the Derivative Works; or,\n          within a display generated by the Derivative Works, if and\n          wherever such third-party notices normally appear. The contents\n          of the NOTICE file are for informational purposes only and\n          do not modify the License. You may add Your own attribution\n          notices within Derivative Works that You distribute, alongside\n          or as an addendum to the NOTICE text from the Work, provided\n          that such additional attribution notices cannot be construed\n          as modifying the License.\n\n      You may add Your own copyright statement to Your modifications and\n      may provide additional or different license terms and conditions\n      for use, reproduction, or distribution of Your modifications, or\n      for any such Derivative Works as a whole, provided Your use,\n      reproduction, and distribution of the Work otherwise complies with\n      the conditions stated in this License.\n\n   5. Submission of Contributions. Unless You explicitly state otherwise,\n      any Contribution intentionally submitted for inclusion in the Work\n      by You to the Licensor shall be under the terms and conditions of\n      this License, without any additional terms or conditions.\n      Notwithstanding the above, nothing herein shall supersede or modify\n      the terms of any separate license agreement you may have executed\n      with Licensor regarding such Contributions.\n\n   6. Trademarks. This License does not grant permission to use the trade\n      names, trademarks, service marks, or product names of the Licensor,\n      except as required for reasonable and customary use in describing the\n      origin of the Work and reproducing the content of the NOTICE file.\n\n   7. Disclaimer of Warranty. Unless required by applicable law or\n      agreed to in writing, Licensor provides the Work (and each\n      Contributor provides its Contributions) on an \"AS IS\" BASIS,\n      WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or\n      implied, including, without limitation, any warranties or conditions\n      of TITLE, NON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A\n      PARTICULAR PURPOSE. You are solely responsible for determining the\n      appropriateness of using or redistributing the Work and assume any\n      risks associated with Your exercise of permissions under this License.\n\n   8. Limitation of Liability. In no event and under no legal theory,\n      whether in tort (including negligence), contract, or otherwise,\n      unless required by applicable law (such as deliberate and grossly\n      negligent acts) or agreed to in writing, shall any Contributor be\n      liable to You for damages, including any direct, indirect, special,\n      incidental, or consequential damages of any character arising as a\n      result of this License or out of the use or inability to use the\n      Work (including but not limited to damages for loss of goodwill,\n      work stoppage, computer failure or malfunction, or any and all\n      other commercial damages or losses), even if such Contributor\n      has been advised of the possibility of such damages.\n\n   9. Accepting Warranty or Additional Liability. While redistributing\n      the Work or Derivative Works thereof, You may choose to offer,\n      and charge a fee for, acceptance of support, warranty, indemnity,\n      or other liability obligations and/or rights consistent with this\n      License. However, in accepting such obligations, You may act only\n      on Your own behalf and on Your sole responsibility, not on behalf\n      of any other Contributor, and only if You agree to indemnify,\n      defend, and hold each Contributor harmless for any liability\n      incurred by, or claims asserted against, such Contributor by reason\n      of your accepting any such warranty or additional liability.\n\n   END OF TERMS AND CONDITIONS\n\n   APPENDIX: How to apply the Apache License to your work.\n\n      To apply the Apache License to your work, attach the following\n      boilerplate notice, with the fields enclosed by brackets \"[]\"\n      replaced with your own identifying information. (Don't include\n      the brackets!)  The text should be enclosed in the appropriate\n      comment syntax for the file format. We also recommend that a\n      file or class name and description of purpose be included on the\n      same \"printed page\" as the copyright notice for easier\n      identification within third-party archives.\n\n   Copyright 2021 Bitcrowd GmbH\n\n   Licensed under the Apache License, Version 2.0 (the \"License\");\n   you may not use this file except in compliance with the License.\n   You may obtain a copy of the License at\n\n       http://www.apache.org/licenses/LICENSE-2.0\n\n   Unless required by applicable law or agreed to in writing, software\n   distributed under the License is distributed on an \"AS IS\" BASIS,\n   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.\n   See the License for the specific language governing permissions and\n   limitations under the License.","ref":"license.html"},{"type":"extras","title":"BitcrowdEcto","doc":"<!-- SPDX-License-Identifier: Apache-2.0 -->\n\n# BitcrowdEcto\n\n<!-- MDOC -->\n\nThis library contains Bitcrowd's collection of tiny Ecto helpers.\n\n* `BitcrowdEcto.Schema` is our default schema template which configures PK/FKs and timestamp types, among other things.\n* `BitcrowdEcto.Repo` contains extensions for Ecto repos, like `fetch/2`.\n* `BitcrowdEcto.Migrator` contains a tool for migrating from within releases.\n* `BitcrowdEcto.Migration` contains functions used in migrations.\n* `BitcrowdEcto.Changeset` contains mostly validators, plus the introspection-based `cast_all/3`.\n  - The `validate_money/3` validator is enabled when you add `ex_money` as a dependency to your project. Note that you need to recompile `bitcrowd_ecto` when you add `ex_money` later.\n* `BitcrowdEcto.DateTime` contains date/time helpers.\n* `BitcrowdEcto.Assertions` has an assortment of useful ExUnit assertions related to Ecto schemas.\n* `BitcrowdEcto.Random` contains functions that generate random tokens.\n\n<!-- MDOC -->","ref":"readme.html"},{"type":"extras","title":"Steps to release - BitcrowdEcto","doc":"* Replace the version number in `mix.exs`\n* Set version number & date in `CHANGELOG.md`\n* Commit, push and open a PR for your release\n* When the PR is merged:\n\n```\ngit tag -a 'vX.X.X'\ngit push --tags\nmix hex.publish\n```","ref":"readme.html#steps-to-release"},{"type":"extras","title":"Changelog","doc":"<!-- SPDX-License-Identifier: Apache-2.0 -->\n\n## [1.0.0] - 2023-12-21\n\nNo changes from v0.17.0.\n\n## [0.17.0] - 2023-12-20\n\n#","ref":"changelog.html"},{"type":"extras","title":"Added - Changelog","doc":"* Add `BitcrowdEcto.Changeset.cast_all/3`, a introspection-based automatic cast function.\n* Add `BitcrowdEcto.Assertions.assert_cast_error_on/2`.\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Fixed - Changelog","doc":"* Allow any type of `id` param in `Repo.fetch/2`. Remove the (incorrect) guard restricting the `id` param to binaries, against the spec saying it would allow `any`.\n* Handle missing schemas gracefully in `Migrator.ensure_up!/0`.\n\n## [0.16.0] - 2023-03-21\n\n#","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"* Convert `Ecto.Query.CastError` in `Repo.fetch_by/3` & friends to `:not_found` errors. Opt-out via `raise_cast_error` option.\n\n## [0.15.0] - 2023-02-13\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Added - Changelog","doc":"* Give `Repo.fetch_by/2`, `Repo.count/1`, and `Assertions.assert_count_difference/4` an optional  `opts` parameter that is forwarded to the underlying Ecto function. Useful to pass query options like `:prefix`.\n\n## [0.14.0] - 2023-01-03\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Added - Changelog","doc":"* Changed `Repo.fetch/2` and `Repo.fetch/3` to work with primary key with name other than `id`.\n\n## [0.13.2] - 2022-12-19\n\n* Add `error_tag` option to `BitcrowdEcto.Repo.fetch_by/3` and friends to allow overriding the queryable in the error result.\n\n## [0.13.1] - 2022-09-21\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Fixed - Changelog","doc":"* Fix dependency specifications by restoring the `:only` options and explicitly listing conflicting packages. See https://github.com/kipcole9/money/issues/142 for explanation.\n\n## [0.13.0] - 2022-09-14\n\nNOTE: 0.13.0 needed to be retired due to a corrupt dependency tree listing all dev/test dependencies as prod dependencies.\n\n* Add `BitcrowdEcto.Changeset.validate_money/3` for validating `Money` values from `ex_money`, which is introduced as an optional dependency.\n\n## [0.12.0] - 2022-06-21\n\n#","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"* Add `BitcrowdEcto.DateTime.beginning_of_month/1`.\n* Add `BitcrowdEcto.DateTime.beginning_of_last_month/1`.\n* Add `BitcrowdEcto.DateTime.beginning_of_next_month/1`.\n* Add `BitcrowdEcto.DateTime.beginning_of_yesterday/1`.\n* Add `BitcrowdEcto.DateTime.beginning_of_tomorrow/1`.\n\n## [0.11.0] - 2022-06-09\n\n* Add `BitcrowdEcto.Changeset.validate_date_after/3` Validates a date field in the changeset is after the given reference date.\n* Add `BitcrowdEcto.Changeset.validate_hex_color/2` validates a changeset field has hexadecimal color format.\n* Add `BitcrowdEcto.Assertions.assert_changeset_valid/1`.\n* Add `BitcrowdEcto.Assertions.refute_changeset_valid/1`.\n\n\n## [0.10.0] - 2022-04-28\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Added - Changelog","doc":"* New `BitcrowdEcto.Assertions.assert_*_constraint_on/2` matchers assert on constraints on changesets without going to the database.\n* Add `BitcrowdEcto.DateTime.shift/2` and `BitcrowdEcto.DateTime.beginning_of_day/1`.\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"* BREAKING: Stopped `inspect`ing the `from`/`to` fields in `validate_transition/3` error details. Previously atom columns (e.g. from `Ecto.Enum`) would result in `[from: \":foo\", to: \":bar\"]` error details, now these atoms are inserted unchanged as `[from: :foo, to: :bar]`\n* BREAKING: Removed `:field` from `validate_transition/3` error details as it is redundant with the error field.\n* Deprecated `BitcrowdEcto.Assertions.assert_foreign_constraint_error_on/2` in favour of new `assert_foreign_key_constraint_error_on/2`. Constraint assert functions are all called after the `Ecto.Changeset.*_constraint` functions now, not after internal error type names.\n\n## [0.9.0] - 2022-03-25\n\n#","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"* Add `BitcrowdEcto.Assertions.assert_change_to_almost_now/2` to assert that the value of a datetime field changed to the present time.\n\n* Add `BitcrowdEcto.Schema.to_enum_member/3` and `BitcrowdEcto.Schema.to_enum_member!/3`, functions that safely convert a string to the member of an enum based on reflection on the given schema.\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"* BREAKING: Add `:only_web` option to `BitcrowdEcto.Changeset.validate_email/3` that enforces a dot in the host part of the email address (e.g. `foo@example.net` is valid, `foo@example` is not). Defaults to **true**.\n\n## [0.8.0] - 2022-03-03\n\n#","ref":"changelog.html#changed"},{"type":"extras","title":"Changed - Changelog","doc":"* BREAKING: Drop the `BitcrowdEcto.Migration.grant_dml_privileges_on_schema/2` variant that reads from a configuration variable. However, the keyword list on `grant_dml_privileges_on_schema/3` has become optional, so effectively there is a different `/2` now.\n\n## [0.7.0] - 2022-03-02\n\n#","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"* Add `BitcrowdEcto.Migration` for migration utilities, first util is `grant_dml_privileges_on_schema/2` that `GRANT`s certain privileges to a Postgres role.\n\n## [0.6.0] - 2022-02-10\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Added - Changelog","doc":"* Add `BitcrowdEcto.Random` containing various random token generators.\n* Add `validate_past_datetime/3`, `validate_future_datetime/3`, `validate_datetime_after/4`, `validate_date_order/4`, `validate_datetime_order/4` and `validate_order/5` to `BitcrowdEcto.Changeset`.\n\n## [0.5.0] - 2022-02-04\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Added - Changelog","doc":"* Add `Migrator.ensure_up!/0` to check whether all migrations are up.\n\n## [0.4.0] - 2022-01-28\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Added - Changelog","doc":"* Add `handle_migrator_exception/2` callback to `Migrator` to allow exception reporting.\n\n## [0.3.1] - 2022-01-17\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Added - Changelog","doc":"* Add missing `assert_sorted_equal/3`.\n\n## [0.3.0] - 2022-01-17\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Added - Changelog","doc":"* Add `assert_almost_now/1`, `assert_almost_coincide/3` and `assert_sorted_equal/2` to `BitcrowdEcto.Assertions`.\n\n## [0.2.0] - 2022-01-17\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Added - Changelog","doc":"* Add `BitcrowdEcto.DateTime.in_seconds/1`.\n\n## [0.1.0] - 2022-01-07\n\nInitial release to the public 🎉","ref":"changelog.html#added"}],"content_type":"text/markdown"}